@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2009-01-09
+ *  Updated   : 2009-12-09
  *  Notes     :
  *
- * Copyright (c) 1997-2009 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2010 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,6 +54,7 @@
 /* active.c */
 extern char group_flag(char ch);
 extern int find_newnews_index(const char *cur_newnews_host);
+extern int read_news_active_file(void);
 extern t_bool match_group_list(const char *group, const char *group_list);
 extern t_bool parse_active_line(char *line, long *max, long *min, char *moderated);
 extern t_bool process_bogus(char *name);
@@ -61,7 +62,6 @@ extern t_bool need_reread_active_file(void);
 extern t_bool resync_active_file(void);
 extern void create_save_active_file(void);
 extern void load_newnews_info(char *info);
-extern void read_news_active_file(void);
 
 /* art.c */
 extern int global_get_multipart_info(int aindex, MultiPartInfo *setme);
@@ -72,10 +72,9 @@ extern void make_threads(struct t_group *group, t_bool rethread);
 extern void set_article(struct t_article *art);
 extern void show_art_msg(const char *group);
 extern void sort_arts(unsigned /* int */ sort_art_type);
-extern void write_overview(struct t_group *group);
 
 /* attrib.c */
-extern int add_tempscope(const char *scope);
+extern int add_scope(const char *scope);
 extern void assign_attributes_to_groups(void);
 extern void build_news_headers_array(struct t_attribute *scope, t_bool header_to_display);
 extern void read_attributes_file(t_bool global_file);
@@ -121,7 +120,7 @@ extern void write_config_file(char *file);
 
 /* cook.c */
 extern const char *get_filename(t_param *ptr);
-extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int tabs, int hide_uue);
+extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int hide_uue);
 extern t_bool expand_ctrl_chars(char **line, size_t *length, size_t cook_width);
 
 /* curses.c */
@@ -177,7 +176,7 @@ extern void word_highlight_string(int row, int col, int size, int color);
 extern void envargs(int *Pargc, char ***Pargv, const char *envstr);
 
 /* feed.c */
-extern void feed_articles(int function, int level, struct t_group *group, int respnum);
+extern int feed_articles(int function, int level, t_function type, struct t_group *group, int respnum);
 
 /* filter.c */
 extern t_bool filter_articles(struct t_group *group);
@@ -211,6 +210,7 @@ extern void set_first_screen_item(void);
 /* group.c */
 extern int find_new_pos(int old_top, long old_artnum, int cur_pos);
 extern int group_page(struct t_group *group);
+extern t_bool group_mark_postprocess(int function, t_function feed_type, int respnum);
 extern void clear_note_area(void);
 extern void mark_screen(int screen_row, int screen_col, const char *value);
 extern void pos_first_unread_thread(void);
@@ -308,6 +308,7 @@ extern void init_alloc(void);
 extern void free_all_arrays(void);
 extern void free_art_array(void);
 extern void free_save_array(void);
+extern void free_scope(int num);
 extern void *my_malloc1(const char *file, int line, size_t size);
 extern void *my_calloc1(const char *file, int line, size_t nmemb, size_t size);
 extern void *my_realloc1(const char *file, int line, void *p, size_t size);
@@ -338,7 +339,7 @@ extern int my_chdir(char *path);
 extern int my_mkdir(char *path, mode_t mode);
 extern int parse_from(const char *from, char *address, char *realname);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
-extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group);
+extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group, t_bool expand_all);
 extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
 extern int tin_version_info(FILE *fp);
 extern long file_mtime(const char *file);
@@ -404,7 +405,7 @@ extern void delete_group(char *group);
 extern void expand_bitmap(struct t_group *group, long min);
 extern void grp_mark_read(struct t_group *group, struct t_article *art);
 extern void grp_mark_unread(struct t_group *group);
-extern void parse_unread_arts(struct t_group *group);
+extern void parse_unread_arts(struct t_group *group, long min);
 extern void reset_newsrc(void);
 extern void subscribe(struct t_group *group, int sub_state, t_bool get_info);
 extern void thd_mark_read(struct t_group *group, long thread);
@@ -413,7 +414,7 @@ extern void set_default_bitmap(struct t_group *group);
 
 /* nntplib.c */
 extern FILE *get_nntp_fp(FILE *fp);
-extern FILE *get_nntp_wr_fp(FILE *fp);
+/* extern FILE *get_nntp_wr_fp(FILE *fp); */
 extern char *getserverbyfile(const char *file);
 extern int nntp_open(void);
 extern void nntp_close(void);
@@ -428,12 +429,14 @@ extern void nntp_close(void);
 #endif /* NNTP_ABLE */
 
 /* nrctbl.c */
-extern int get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const char *nntpserver_name);
-extern void get_nntpserver(char *nntpserver_name, size_t nntpserver_name_len, char *nick_name);
+extern t_bool get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const char *nntpserver_name);
+#ifdef NNTP_ABLE
+	extern void get_nntpserver(char *nntpserver_name, size_t nntpserver_name_len, char *nick_name);
+#endif /* NNTP_ABLE */
 
 /* options_menu.c */
 extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, enum option_enum option);
-extern void change_config_file(struct t_group *group);
+extern void config_page(const char *grpname);
 extern int option_row(enum option_enum option);
 extern t_bool option_is_visible(enum option_enum option);
 extern void check_score_defaults(void);
@@ -483,7 +486,7 @@ extern t_bool user_posted_messages(void);
 extern void init_postinfo(void);
 extern void quick_post_article(t_bool postponed_only);
 #ifdef USE_CANLOCK
-	extern const char *build_canlock(const char *messageid, const char *secret);
+	extern char *build_canlock(const char *messageid, const char *secret);
 	extern char *get_secret(void);
 #endif /* USE_CANLOCK */
 
@@ -503,6 +506,7 @@ extern t_bool prompt_option_string(enum option_enum option);
 extern t_bool prompt_string(const char *prompt, char *buf, int which_hist);
 extern void prompt_continue(void);
 extern void prompt_slk_redraw(void);
+extern void prompt_yn_redraw(void);
 
 /* read.c */
 extern char *tin_fgets(FILE *fp, t_bool header);
@@ -673,10 +677,8 @@ extern void str_lwr(char *str);
 
 /* tags.c */
 extern int line_is_tagged(int n);
-extern int mark_tagged_read(struct t_group *group);
 extern int tag_multipart(int base_index);
 extern t_bool arts_selected(void);
-extern t_bool got_tagged_unread_arts(void);
 extern t_bool set_range(int level, int min, int max, int curr);
 extern t_bool tag_article(int art);
 extern t_bool untag_all_articles(void);
@@ -707,6 +709,7 @@ extern int stat_thread(int n, struct t_art_stat *sbuf);
 extern int which_response(int n);
 extern int which_thread(int n);
 extern int thread_page(struct t_group *group, int respnum, int thread_depth, t_pagerinfo *page);
+extern t_bool thread_mark_postprocess(int function, t_function feed_type, int respnum);
 extern void fixup_thread(int respnum, t_bool redraw);
 
 /* version.c */
