@@ -3,12 +3,12 @@
  *  Module    : refs.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 2008-12-04
+ *  Updated   : 2009-11-17
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2009 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1996-2010 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,10 +56,11 @@ static char *_get_references(struct t_msgid *refptr, int depth);
 static struct t_msgid *add_msgid(int key, const char *msgid, struct t_msgid *newparent);
 static struct t_msgid *find_next(struct t_msgid *ptr);
 static struct t_msgid *parse_references(char *r);
-static t_bool valid_msgid(const char *msgid);
+static t_bool valid_msgid(char *msgid);
 static unsigned int hash_msgid(const char *key);
 static void add_to_parent(struct t_msgid *ptr);
 static void build_thread(struct t_msgid *ptr);
+static void rearrange_siblings(void);
 #ifdef DEBUG
 	static void dump_msgid_thread(struct t_msgid *ptr, int level);
 	static void dump_msgid_threads(void);
@@ -69,11 +70,6 @@ static void build_thread(struct t_msgid *ptr);
 	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
 #endif /* 0 */
 
-/*
- * Set if the sorting algorithm goes 'upwards'
- */
-static t_bool sort_ascend;
-
 /*
  * The msgids are all hashed into a big array, with overspill
  */
@@ -148,27 +144,14 @@ add_to_parent(
 
 	/*
 	 * Add this followup to the sibling chain of our parent.
-	 * arts[] has been sorted by build_references and we add at the start or end
-	 * of the chain depending on whether the sort method is ASCEND or DESCEND
-	 * Unavailable articles go at the start of the chain if ASCEND (because
-	 * we presume unavailable arts (ie REF_REF links) have expired), otherwise at the end.
-	 * ie: if ASCEND && REF
-	 *        add_to_start
-	 *     else
-	 *        add_to_end
+	 * We add at the end of the chain, rearrange_siblings() will
+	 * sort it later.
 	 */
-	if (sort_ascend && (ptr->article == ART_UNAVAILABLE)) {
-		/* Add to start */
-		ptr->sibling = ptr->parent->child;
-		ptr->parent->child = ptr;
-	} else {
-		/* Add to end */
-		for (p = ptr->parent->child; p->sibling != NULL; p = p->sibling)
-			;
+	for (p = ptr->parent->child; p->sibling != NULL; p = p->sibling)
+		;
 
-/*		ptr->sibling is already NULL */
-		p->sibling = ptr;
-	}
+/*	ptr->sibling is already NULL */
+	p->sibling = ptr;
 }
 
 
@@ -176,15 +159,16 @@ add_to_parent(
  * Checks if Message-ID has valid format
  * Returns TRUE if it does, FALSE if it does not
  *
- * TODO: combine with post.c:damaged_id()?
+ * TODO: combine with post.c:damaged_id()
  */
 static t_bool
 valid_msgid(
-	const char *msgid)
+	char *msgid)
 {
 	size_t mlen = 0;
 	t_bool at_present = 0;
 
+	str_trim(msgid);
 	if (!msgid || *msgid != '<')
 		return FALSE;
 
@@ -432,6 +416,60 @@ parse_references(
 }
 
 
+static void
+rearrange_siblings(
+	void)
+{
+	int i;
+	struct t_msgid *current, *p1, *p2;
+
+	for_each_art(i) {
+		current = arts[i].refptr;
+
+		if (!current)
+		continue;
+
+		for (; current->sibling == NULL && current->parent != NULL; current = current->parent)
+			;
+
+		if (current->sibling != NULL) {
+			for (p1 = current; p1->article == ART_UNAVAILABLE && p1->child != NULL; p1 = p1->child)
+				;
+
+			for (p2 = current->sibling; p2->article == ART_UNAVAILABLE && p2->child != NULL; p2 = p2->child)
+				;
+
+			if (p1->article != ART_UNAVAILABLE && p2->article != ART_UNAVAILABLE && p1->article > p2->article) {
+				if (current->parent->child == current) {
+					/*
+					 * current is the first followup
+					 *  adjust parent->child pointer
+					 */
+					current->parent->child = current->sibling;
+					p1 = current->parent->child;
+				} else {
+					/*
+					 * current is not the first followup
+					 *  find the sibling above current
+					 *  adjust the sibling pointer there
+					 */
+					for (p1 = current->parent->child; p1->sibling != current; p1 = p1->sibling)
+						;
+
+					p1->sibling = current->sibling;
+					p1 = p1->sibling;
+				}
+				/*
+				 * swap current <-> sibling
+				 */
+				current->sibling = p1->sibling;
+				p1->sibling = current;
+			}
+		}
+	}
+}
+
+
 /*
  * Reconstruct the References: field from the parent pointers
  * NB: In deep threads this can lead to a very long line. If you want to use
@@ -944,12 +982,6 @@ build_references(
 	if (group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
 		sort_arts(group->attribute->sort_article_type);
 
-	sort_ascend = (group->attribute->sort_article_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
-	               group->attribute->sort_article_type == SORT_ARTICLES_BY_FROM_ASCEND ||
-	               group->attribute->sort_article_type == SORT_ARTICLES_BY_DATE_ASCEND ||
-	               group->attribute->sort_article_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
-	               group->attribute->sort_article_type == SORT_ARTICLES_BY_LINES_ASCEND);
-
 #ifdef DEBUG
 	if (debug & DEBUG_REFS) {
 		char file[PATH_LEN];
@@ -968,6 +1000,7 @@ build_references(
 	for_each_art(i) {
 		art = &arts[i];
 
+		art->refptr = add_msgid(MSGID_REF, art->msgid, NULL); /* preset art->refptr */
 		if (art->refs) {
 			strip_line(art->refs);
 
@@ -999,9 +1032,14 @@ build_references(
 					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
 				FreeAndNull(art->refs);
 			}
-		} else
-			if (valid_msgid(art->msgid))
-				art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
+		}
+
+		/*
+		 * set art->refptr->article - rearrange_siblings() needs this
+		 */
+		if (art->refptr != NULL)
+			art->refptr->article = i;
+
 		FreeAndNull(art->msgid);	/* Now cached - discard this */
 	}
 
@@ -1035,4 +1073,10 @@ build_references(
 	if (debug & DEBUG_REFS)
 		fclose(dbgfd);
 #endif /* DEBUG */
+
+	/*
+	 * all msgids are cached now
+	 * change order of siblings if needed
+	 */
+	rearrange_siblings();
 }
