@@ -3,10 +3,10 @@
  *  Module    : xref.c
  *  Author    : I. Lea & H. Brugge
  *  Created   : 1993-07-01
- *  Updated   : 2009-01-15
+ *  Updated   : 2009-04-09
  *  Notes     :
  *
- * Copyright (c) 1993-2009 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2010 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,9 +59,8 @@ open_overview_fmt_fp(
 {
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && !read_saved_news) {
-		if (!nntp_caps.over_cmd)
+		if (!*nntp_caps.over_cmd)
 			return (FILE *) 0;
-
 		if ((nntp_caps.type == CAPABILITIES && nntp_caps.list_overview_fmt) || nntp_caps.type != CAPABILITIES)
 			return (nntp_command("LIST OVERVIEW.FMT", OK_GROUPS, NULL, 0));
 		else
@@ -87,7 +86,6 @@ overview_xref_support(
 	size_t fields = 0;
 	size_t i;
 
-	ofmt = my_malloc(sizeof(*ofmt) * res_fields);
 	ofmt[0].type = OVER_T_INT;
 	ofmt[0].name = my_strdup("Artnum:");
 
